# 下記2行はエラー時にスタックトレースを表示したい時のみ記述
from bottle import debug
debug(True)

from collections import namedtuple
from abalone_predictor import AbalonePredictor
from bokeh.plotting import figure
from bokeh.embed import components
from bottle import (
    default_app, route, template,
    request, HTTPError
)


READABLE_SEX = ['メス', '不明', 'オス']

# ユーザー入力のパラメータは不変なためtupleで表現
INPUT_DATA = (
    'sex', 'length', 'diameter', 'height', 'weight')

# Abaloneはユーザー入力のパラメータと
# 年齢を属性として持つクラス
BaseAbalone = namedtuple(
    'BaseAbalone', INPUT_DATA + ('age', ))
class Abalone(BaseAbalone):
    # propertyデコレータを利用して
    # ドット記法でアクセスできるようにする
    @property
    def sex_str(self):
        return READABLE_SEX[int(self.sex)]


@route('/')
def index():
    return template('templates/index.tpl')


@route('/abalone', method='POST')
def result():
    try:
        # 辞書のアンパック
        age = calc_age(**request.params)
        abalone = Abalone(age=age, **request.params)
    except (TypeError, ValueError) as e:
        raise HTTPError(status=400, body=e)
    else:
        script, div = get_graph(abalone)
        return template('templates/result.tpl',
                        abalone=abalone, script=script, graph=div)


# インスタンス化はモジュールの読み込み時のみ行う
_predictor = AbalonePredictor()

def calc_age(sex, length, diameter, height, weight):
    """アワビの年齢を推定する"""
    age = _predictor.predict(
        int(sex),
        int(length),
        int(diameter),
        int(height),
        int(weight))
    return float(age)


def get_graph(abalone):
    p = figure(plot_width=400,
               plot_height=400,
               title='実年齢と推定値の分布')
    p.xaxis.axis_label = '実年齢'
    p.yaxis.axis_label = '推定値'

    # 誤差がわかりやすいようにする
    p.line([0, 30], [0, 30],
           line_dash='dotted',
           legend='実年齢と推定値が一致するライン')

    # 実年齢と推定値のデータをプロット
    p.circle(_predictor.y_train,
             _predictor.prediction,
             legend='訓練データにおける分布')

    # ブラウザから入力したアワビの推定値をプロット
    p.line([0, 30], [abalone.age, abalone.age],
           legend="捕まえたアワビの推定年齢",
           color="green")

    # 凡例の表示位置を設定
    p.legend.location = 'top_left'
    # 凡例クリックでグラフと凡例を半透明にする
    p.legend.click_policy='mute'

    # JavaScriptコードが生成される
    script, div = components(p)
    return script, div

application = default_app()
